<?php namespace Myriad\SharePoint\Auth;

use Myriad\SharePoint\Contracts\SharePointClient;
use Myriad\SharePoint\Contracts\TokenRetriever;
use Myriad\SharePoint\Models\Token\Token;
use \Rhumsaa\Uuid\Uuid;
use Illuminate\Support\Facades\Log;

class AdfsClaimAuthenticator implements TokenRetriever {

  /**
   * The realm to use when requesting an OAuth token.
   */
  const REALM = "urn:federation:MicrosoftOnline";

  /**
   * The grant type to use when requesting an OAuth token.
   */
  const GRANT_TYPE = "urn:ietf:params:oauth:grant-type:saml1_1-bearer";

  /**
   * The scope to use when requesting an OAuth token.
   */
  const SCOPE = "AllSites.Read AllProfiles.Read Search.QueryAsUserIgnoreAppPrincipal";

  /**
   * The format of the security token request.
   *
   * @var string
   */
  const SECURITY_TOKEN_REQUEST_TEMPLATE = "<s:Envelope xmlns:s='http://www.w3.org/2003/05/soap-envelope' xmlns:a='http://www.w3.org/2005/08/addressing' xmlns:u='http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd'><s:Header><a:Action s:mustUnderstand='1'>http://schemas.xmlsoap.org/ws/2005/02/trust/RST/Issue</a:Action><a:MessageID>urn:uuid:%s</a:MessageID><a:ReplyTo><a:Address>http://www.w3.org/2005/08/addressing/anonymous</a:Address></a:ReplyTo><a:To s:mustUnderstand='1'>%s</a:To><o:Security s:mustUnderstand='1' xmlns:o='http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd'><o:UsernameToken u:Id='uuid-%s-1'><o:Username>%s</o:Username><o:Password Type='http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-username-token-profile-1.0#PasswordText'>%s</o:Password></o:UsernameToken></o:Security></s:Header><s:Body><t:RequestSecurityToken xmlns:t='http://schemas.xmlsoap.org/ws/2005/02/trust'><wsp:AppliesTo xmlns:wsp='http://schemas.xmlsoap.org/ws/2004/09/policy'><wsa:EndpointReference xmlns:wsa='http://www.w3.org/2005/08/addressing'><wsa:Address>%s</wsa:Address></wsa:EndpointReference></wsp:AppliesTo><t:KeyType>http://schemas.xmlsoap.org/ws/2005/05/identity/NoProofKey</t:KeyType><t:RequestType>http://schemas.xmlsoap.org/ws/2005/02/trust/Issue</t:RequestType></t:RequestSecurityToken></s:Body></s:Envelope>";

  /**
   * @var string
   */
  private $username;

  /**
   * @var string
   */
  private $password;

  /**
   * @var string
   */
  private $adfsHost;

  /**
   * @var string
   */
  private $clientID;

  /**
   * @var \GuzzleHttp\Client
   */
  private $httpClient;

  /**
   * @var SharePointClient
   */
  private $spoClient;

  /**
   * @var string
   */
  private $tenant;

  /**
   * AdfsClaimAuthenticator constructor.
   *
   * @param $username string The username of the user to create the OAuth token for.
   * @param $password string The password of the user to create the OAuth token for.
   * @param $adfsHost string The URL of the ADFS endpoint to get the assertion from.
   * @param $tenant string The SharePoint Online tenant.
   * @param $clientID string The OAuth clientID to generate the token against.
   * @param $spoClient SharePointClient The SharePointOnline client.
   * @param \GuzzleHttp\Client $httpClient The HTTP client to make requests with.
   */
  public function __construct($username, $password, $adfsHost, $tenant, $clientID, $spoClient, $httpClient) {

    $this->username = $username;
    $this->password = $password;
    $this->adfsHost = $adfsHost;
    $this->tenant = $tenant;
    $this->clientID = $clientID;
    $this->httpClient = $httpClient;
    $this->spoClient = $spoClient;
  }

  /**
   * Hydrates the security token with the necessary data.
   *
   * @param $adfsEndpoint string The ADFS endpoint to make the request against.
   * @param $username string The username to login to ADFS with.
   * @param $password string The password to login to ADFS with.
   *
   * @return string
   */
  private function hydrateSecurityTokenTemplate($adfsEndpoint, $username, $password) {
    $messageId = Uuid::uuid4();
    $usernameTokenId = Uuid::uuid4();

    $loggable = sprintf(
      self::SECURITY_TOKEN_REQUEST_TEMPLATE,
      $messageId,
      $adfsEndpoint,
      $usernameTokenId,
      $username,
      '***redacted***',
      self::REALM
    );

    log::error(__METHOD__ . " " . __LINE__ . "Generated securityToken: " . $loggable);

    return sprintf(
      self::SECURITY_TOKEN_REQUEST_TEMPLATE,
      $messageId,
      $adfsEndpoint,
      $usernameTokenId,
      $username,
      $password,
      self::REALM
    );
  }

  /**
   * Request the security token from the ADFS server.
   *
   * @param $requestSecurityTokenXml string The XML to send to the server when requesting a security token.
   * @param $adfsEndpoint string The ADFS endpoint to make the authentication request to.
   * @return string
   */
  private function requestAdfsAssertion($requestSecurityTokenXml, $adfsEndpoint) {
    $response = $this->httpClient->post(
      $adfsEndpoint,
      ['body' => $requestSecurityTokenXml, 'headers' => ['content-type' => 'application/soap+xml; charset=utf-8']]
    );

    $adfsXml = $response->getBody()->getContents();

    return $adfsXml;
  }

  /**
   * Extracts the Assertion from the response to ADFS request for a security token.
   *
   * @param string $adfsAssertionResponse The response from ADFS for a security token.*
   * @return string
   */
  private function extractAdfsAssertion($adfsAssertionResponse) {
    $xml = new \SimpleXMLElement($adfsAssertionResponse);
    $xml->registerXPathNamespace('t', 'http://schemas.xmlsoap.org/ws/2005/02/trust');
    $xml->registerXPathNamespace('saml', 'urn:oasis:names:tc:SAML:1.0:assertion');

    $xpathQuery = '/s:Envelope/s:Body/t:RequestSecurityTokenResponse/t:RequestedSecurityToken/saml:Assertion';
    $src = $xml->xpath($xpathQuery);

    $adfsAssertion = '';
    while (list(, $node) = each($src)) {
      $adfsAssertion = $node->asXML();
    }

    return $adfsAssertion;
  }

  /**
   * Request a token from Microsoft for mastec.
   *
   * @param $adfsAssertion string The ADFS assertion returned from the ADFS endpoint.
   * @param $sharePointUrl string The URL of the SharePoint site.
   * @param $clientID string The clientID needed to generate the OAuth token against.
   * @param $msOAuthEndpoint string The URL of the Microsoft Online OAuth endpoint.
   * @return \stdClass
   */
  public function requestOAuthToken($adfsAssertion, $sharePointUrl, $clientID, $msOAuthEndpoint) {
    $encodedAssertion = base64_encode($adfsAssertion);

    $body = [
      'resource' => $sharePointUrl,
      'client_id' => $clientID,
      'grant_type' => $this::GRANT_TYPE,
      'assertion' => $encodedAssertion,
      'scope' => $this::SCOPE
    ];

    $response = $this->httpClient->post(
      $msOAuthEndpoint,
      [
        'body' => $body,
        'allow_redirects' => false,
        'exceptions' => false
      ]
    );

    $tokenResponse = $response->getBody()->getContents();

    $decodedResponse = json_decode($tokenResponse, true);

    if ($this->returnedTokensAreInvalid($decodedResponse)) {
      log::error(__METHOD__ . " " . __LINE__ . " Invalid OAuthToken in response.  Status code: " . $response->getStatusCode() );
      log::error(__METHOD__ . " " . __LINE__ . " Body: " . $response->getBody() );
    }

    return $decodedResponse;
  }

  /**
   * Checks to see if the tokens returned from SharePointOnline are invalid.
   *
   * @param string $tokens The tokens returned from SharePointOnline.
   *
   * @return bool
   */
  private function returnedTokensAreInvalid($token) {
    $is_invalid = false;

    if( is_null($token) ) {
      $is_invalid = true;
    }

    $required_keys = ['access_token', 'refresh_token', 'not_before', 'expires_on'];
    foreach ($required_keys as $key) {
      if( !array_key_exists($key, $token) ) {
        $is_invalid = true;
      }
    }

    return $is_invalid;
  }


  /**
   * Requests an access token from ADFS.
   */
  public function getAccessToken() {
    $username = $this->username;
    $password = $this->password;
    $clientID = $this->clientID;
    $tenant = $this->tenant;
    $adfsHost = $this->adfsHost;


    // Configure URLs
    $sharePointUrl = "https://{$tenant}.sharepoint.com";
    $adfsEndpoint = "$adfsHost/adfs/services/trust/2005/usernamemixed";
    $msOAuthEndpoint = "https://login.windows.net/{$tenant}.onmicrosoft.com/oauth2/token";

    // Fetch token
    $securityToken = $this->hydrateSecurityTokenTemplate($adfsEndpoint, $username, $password);
    $adfsAssertionResponse = $this->requestAdfsAssertion($securityToken, $adfsEndpoint);
    $adfsAssertion = $this->extractAdfsAssertion($adfsAssertionResponse);
    $token = $this->requestOAuthToken($adfsAssertion, $sharePointUrl, $clientID, $msOAuthEndpoint);

    return new Token(
      $token['access_token'],
      $token['refresh_token'],
      $token['not_before'],
      $token['expires_on'],
      ""
    );
  }
}