<?php namespace Myriad\SharePoint\Auth;

use GuzzleHttp\Cookie\CookieJar;
use GuzzleHttp\Message\MessageInterface;
use Myriad\SharePoint\Auth\SharePointOnline\Operations;
use Myriad\SharePoint\Contracts\TokenRetriever;
use Myriad\SharePoint\Models\Token\Token;
use Myriad\SharePoint\SharePointClient;

/**
 * Class SharePointOnlineClaimAuthenticator Knows how to authenticator to SharePointOnline.
 * http://paulryan.com.au/2014/spo-remote-authentication-rest/
 * http://spshell.blogspot.in/2015/03/sharepoint-online-o365-oauth.html
 *
 * @author David Kanenwisher
 * @package Myriad\SharePoint\Auth
 */
class SharePointOnlineClaimAuthenticator extends Authenticator implements TokenRetriever {

  /**
   * @var string The username of the user who is authenticating.
   */
  private $username;

  /**
   * @var string The password of the user who is authenticating.
   */
  private $password;

  /**
   * @var \GuzzleHttp\Client The guzzle client used to make HTTP requests.
   */
  private $guzzle;

  /**
   * @var array The authenticator's credentials.
   */
  private $credentials = [];

  /**
   * @var string The OAuth2 client ID.
   */
  private $clientId;

  /**
   * @var string the OAuth2 client secret.
   */
  private $clientSecret;

  /**
   * @var string The URI to redirect the client to after authenticating.
   */
  private $redirectUri;

  /**
   * @var SharePointClient Used to get information about the SharePoint instance to make auth requests to.
   */
  private $client;

  /**
   * @var \Myriad\SharePoint\Auth\SharePointOnline\Operations The operations to perform while authenticating.
   */
  private $operations;

  /**
   * Creates a new object.
   *
   * @param string $username The username of the user to authenticate.
   * @param string $password The user's password.
   * @param string $clientId The OAuth2 client ID.
   * @param string $clientSecret The OAuth2 client secret.
   * @param string $redirectUri THe URI to redirect after authenticating.
   * @param SharePointClient $client The SharePointOnline client.
   * @param \GuzzleHttp\Client $guzzle The HTTP client to make requests with.
   * @param \Myriad\SharePoint\Auth\SharePointOnline\Operations $operations The authentication operations.
   */
  public function __construct(
    $username,
    $password,
    $clientId,
    $clientSecret,
    $redirectUri,
    $client,
    $guzzle,
    Operations $operations
  ) {
    $this->username = $username;
    $this->password = $password;
    $this->clientId = $clientId;
    $this->clientSecret = $clientSecret;
    $this->redirectUri = $redirectUri;
    $this->guzzle = $guzzle;
    $this->client = $client;
    $this->operations = $operations;
  }

  /**
   * Returns the SharePointClient.
   *
   * @return SharePointClient
   */
  public function getClient() {
    return $this->client;
  }

  /**
   * Returns security token from SharePoint.
   *
   * @return string
   */
  private function getSecurityToken() {
    if (empty($this->credentials['security_token'])) {
      $this->credentials['security_token'] = $this->fetchSecurityToken();
    }

    return $this->credentials['security_token'];
  }

  /**
   * Retrieves the security token from SharePoint.
   *
   * @return string
   * @throws \Exception
   */
  private function fetchSecurityToken() {
    $saml = $this->generateSamlRequest($this->username, $this->password, $this->getClient()->getEndpoint());
    $response = $this->guzzle->post('https://login.microsoftonline.com/extSTS.srf', ['body' => $saml]);
    $token = $this->operations->getBinarySecurityTokenParser($response->getBody()->getContents())->parse();

    return $token;
  }

  /**
   * Returns the authorization cookies from SharePointOnline.
   *
   * @return array
   */
  private function getAuthCookies() {
    if (empty($this->credentials['cookies'])) {
      $this->credentials['cookies'] = $this->fetchAuthCookies($this->getSecurityToken());
    }

    return $this->credentials['cookies'];
  }

  /**
   * Uses the security token to retrieve authorization cookies from SharePointOnline.
   *
   * @param string $securityToken The security token fetch cookies for.
   *
   * @return \GuzzleHttp\Cookie\CookieJar
   */
  public function fetchAuthCookies($securityToken) {
    $url = $this->getClient()->getEndpoint('_forms', 'default.aspx?wa=wsignin1.0');

    $cookieJar = new CookieJar();
    $this->guzzle->post($url, ['body' => $securityToken, 'allow_redirects' => false, 'cookies' => $cookieJar]);

    return $cookieJar;
  }

  /**
   * Contacts SharePointOnline to retrieve an access token.
   *
   * @return Token
   */
  public function getAccessToken() {
    $url = $this->getClient()->getEndpoint($this->getClient()->getSubsite(), '_layouts', '15', 'OAuthAuthorize.aspx');
    $query = [
      'client_id' => $this->clientId,
      'scope' => 'AllSites.Read AllProfiles.Read Search.QueryAsUserIgnoreAppPrincipal',
      'response_type' => 'code',
      'redirect_uri' => 'https://localhost',
      'IsDlg' => '1'
    ];
    $response = $this->guzzle->post($url, ['cookies' => $this->getAuthCookies(), 'query' => $query, 'allow_redirects' => false, 'exceptions' => false]);

    $htmlConverter = $this->operations->getHtmlToXpath($response->getBody()->getContents());
    $claimForm = $this->operations->getAuthForm($htmlConverter->xpath());

    # grant the request
    $response = $this->guzzle->post($this->getClient()->getEndpoint($this->getClient()->getSubsite(), '_layouts', '15', $claimForm->getAction()), ['cookies' => $this->getAuthCookies(), 'body' => $claimForm->getBody(), 'allow_redirects' => false]);
    $location = $response->getHeader('Location');
    $codeFinder = $this->operations->getCodeFinder($location);

    # get the tokens
    $realm = $this->getRealm();
    $url = 'https://accounts.accesscontrol.windows.net/' . $realm . '/tokens/OAuth/2';
    $body = [
      'grant_type' => 'authorization_code',
      'client_id' => $this->clientId . '@' . $realm,
      'client_secret' => $this->clientSecret,
      'code' => $codeFinder->findCode(),
      'redirect_uri' => $this->redirectUri,
      'resource' => '00000003-0000-0ff1-ce00-000000000000/' . $this->getClient()->getSiteHost() . '@' . $realm
    ];
    $response = $this->guzzle->post($url, ['body' => $body, 'allow_redirects' => false, 'exceptions' => false]);
    $tokens = $response->json();

    return new Token(
      $tokens['access_token'],
      $tokens['refresh_token'],
      $tokens['not_before'],
      $tokens['expires_on'],
      $realm
    );
  }

  /**
   * Returns the realm from the authentication header.
   *
   * @return string
   */
  private function getRealm() {
    $url = $this->getClient()->getEndpoint('_vti_bin', 'client.svc');
    $response = $this->guzzle->get($url, ['headers' => ['Authorization' => 'Bearer'], 'exceptions' => false]);
    $auth = $response->getHeader('WWW-Authenticate');
    $realmFinder = $this->operations->getRealmFinder($auth);

    return $realmFinder->findRealm();
  }

  /**
   * Signs a request.
   *
   * @param \GuzzleHttp\Message\MessageInterface $request The request to sign.
   *
   * @return string
   */
  public function sign(MessageInterface $request) {

  }

  /**
   * Generates the SAML used to authorize the request to SharePointOnline.
   *
   * @param string $username The username of the user to authenticate.
   * @param string $password The password associated with the user.
   * @param string $endpoint The endpoint to authenticate to.
   *
   * @return string
   */
  private function generateSamlRequest($username, $password, $endpoint) {
    return <<<TOKEN
<s:Envelope xmlns:s="http://www.w3.org/2003/05/soap-envelope"
      xmlns:a="http://www.w3.org/2005/08/addressing"
      xmlns:u="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd">
  <s:Header>
    <a:Action s:mustUnderstand="1">http://schemas.xmlsoap.org/ws/2005/02/trust/RST/Issue</a:Action>
    <a:ReplyTo>
      <a:Address>http://www.w3.org/2005/08/addressing/anonymous</a:Address>
    </a:ReplyTo>
    <a:To s:mustUnderstand="1">https://login.microsoftonline.com/extSTS.srf</a:To>
    <o:Security s:mustUnderstand="1"
       xmlns:o="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd">
      <o:UsernameToken>
        <o:Username><![CDATA[$username]]></o:Username>
        <o:Password><![CDATA[$password]]></o:Password>
      </o:UsernameToken>
    </o:Security>
  </s:Header>
  <s:Body>
    <t:RequestSecurityToken xmlns:t="http://schemas.xmlsoap.org/ws/2005/02/trust">
      <wsp:AppliesTo xmlns:wsp="http://schemas.xmlsoap.org/ws/2004/09/policy">
        <a:EndpointReference>
          <a:Address><![CDATA[$endpoint]]></a:Address>
        </a:EndpointReference>
      </wsp:AppliesTo>
      <t:KeyType>http://schemas.xmlsoap.org/ws/2005/05/identity/NoProofKey</t:KeyType>
      <t:RequestType>http://schemas.xmlsoap.org/ws/2005/02/trust/Issue</t:RequestType>
      <t:TokenType>urn:oasis:names:tc:SAML:1.0:assertion</t:TokenType>
    </t:RequestSecurityToken>
  </s:Body>
</s:Envelope>
TOKEN;
  }
}