<?php namespace Myriad\SharePoint\Models;

use Myriad\SharePoint\Contracts;
use Myriad\SharePoint\Contracts\Gateway;

/**
 * Class User represents the user returned by SharePoint and the operations they can perform.
 *
 * @author David Kanenwisher
 * @package Myriad\SharePoint\Model
 */
class AuthenticatedUser implements Contracts\Models\AuthenticatableUser {

  /**
   * @var string The user's e-mail address.
   */
  private $email;

  /**
   * @var \Myriad\SharePoint\Contracts\Gateway Knows how to access SharePointOnline.
   */
  private $gateway;

  /**
   * @var string
   */
  private $name;

  /**
   * @var string
   */
  private $username;

  /**
   * Creates a new user object.
   *
   * @param \Myriad\SharePoint\Contracts\Gateway $gateway Knows how to access SharePointOnline.
   **/
  public function __construct(Gateway $gateway) {
    $this->email = null;
    $this->name = null;
    $this->username = null;
    $this->gateway = $gateway;
  }

  /**
   * Grabs the user's data from SharePointOnline.
   *
   * @return void
   */
  private function hydrate() {
    $currentUser = $this->gateway->getCurrentUser();

    $this->email = $currentUser->getEmailAddress();
    $this->name = $currentUser->getName();
    $this->username = $currentUser->getUsername();
  }

  /**
   * Whether or not the user is authenticated.
   *
   * @return bool
   */
  public function isAuthenticated() {
    return $this->gateway->isAuthenticated();
  }

  /**
   * Extracts the ADFS name from the e-mail address.
   *
   * @return string
   */
  public function getAdfsNameFromEmailAddress() {
    $username = substr($this->getEmailAddress(), 0, strpos($this->getEmailAddress(), '@'));

    return $username . config('sharepoint.user_domain');
  }

  /**
   * The user's e-mail address.
   *
   * @return string
   */
  public function getEmailAddress() {
    if ($this->email === null) {
      $this->hydrate();
    }

    return $this->email;
  }

  /**
   * The user's name.
   *
   * @return string
   */
  public function getName() {
    if ($this->name === null) {
      $this->hydrate();
    }

    return $this->name;
  }

  /**
   * The username the user uses to login to SharePoint Online.
   *
   * @return string
   */
  public function getUsername() {
    if ($this->username === null) {
      $this->hydrate();
    }

    return $this->username;
  }

  /**
   * The user's access token.
   *
   * @return string
   */
  public function getAccessToken() {
    return $this->gateway->getAccessToken();
  }

  /**
   * Returns the user's refresh token.
   *
   * @return string
   */
  public function getRefreshToken() {
    return $this->gateway->getRefreshToken();
  }

  /**
   * Returns the user's expires in time.
   *
   * @return string
   */
  public function getExpiresOn() {
    return $this->gateway->getExpiresOn();
  }

  /**
   * Make a get request to SharePointOnline.
   *
   * @param string $resource The resource to request.
   * @param string $site The SharePoint Online site to make the request to.
   *
   * @return \GuzzleHttp\Message\Response
   */
  public function get($resource, $site = '') {
    return $this->gateway->get($resource, $site);
  }

  /**
   * Make a request to the SharePointOnline search API.
   *
   * @param string $url The url to append to the search request.
   * @return \GuzzleHttp\Message\Response
   * @throws \Exception
   */
  public function search($url) {
    return $this->gateway->search($url);
  }

  /**
   * Get the name of the unique identifier for the user.
   *
   * @return string
   */
  public function getAuthIdentifierName() {
    // not implemented
  }

  /**
   * Get the unique identifier for the user.
   *
   * @return mixed
   */
  public function getAuthIdentifier() {
    // not implemented
  }

  /**
   * Get the password for the user.
   *
   * @return string
   */
  public function getAuthPassword() {
    // not implemented
  }

  /**
   * Get the token value for the "remember me" session.
   *
   * @return string
   */
  public function getRememberToken() {
    // not implemented
  }

  /**
   * Set the token value for the "remember me" session.
   *
   * @param  string $value
   *
   * @return void
   */
  public function setRememberToken($value) {
    // note implemented
  }

  /**
   * Get the column name for the "remember me" token.
   *
   * @return string
   */
  public function getRememberTokenName() {
    // not implemented
  }

  /**
   * Refreshes the user's access token.
   *
   * @return bool
   */
  public function refreshAccessToken() {
    return $this->gateway->refreshAccessToken();
  }
}