<?php namespace Myriad\SharePoint\Models\Token;

use Myriad\SharePoint\Contracts\Models\Token as TokenContract;

/**
 * Class AccessTokenConverter knows how to create a Token from an access token.
 *
 * @package Myriad\SharePoint\Models\Token
 */
class AccessTokenConverter {

  /**
   * @var string The access token.
   */
  private $accessToken;

  /**
   * @var string The refresh token.
   */
  private $refreshToken;

  /**
   * Creates a new object.
   *
   * @param string $accessToken The access token to convert.
   */
  public function __construct($accessToken, $refreshToken = "") {
    $this->accessToken = $accessToken;
    $this->refreshToken = $refreshToken;
  }

  /**
   * Base64 decodes the acess token.
   *
   * @param string $accessToken The access token.
   *
   * @return string
   */
  private function base64Decode($accessToken) {
    return base64_decode($accessToken);
  }

  /**
   * Converts JSON encoded string to an object.
   *
   * @param string $jsonEncodedToken The JSON encoded string that holds the token.
   *
   * @return \stdClass
   */
  private function jsonDecode($jsonEncodedToken) {
    return json_decode($jsonEncodedToken);
  }

  /**
   * Breaks the exploded token into separate pieces since the first 2 pieces are JSON encodded.
   *
   * @param string $binaryToken The binary token to explode.
   *
   * @return array
   */
  private function explodeToken($binaryToken) {
    return explode('}', $binaryToken);
  }

  /**
   * Returns the JSON encoded data that holds the 'aud' section.
   *
   * @param array $explodedToken A token that has been exploded into separate peices.
   *
   * @return mixed
   */
  private function findJsonWithAudInExplodedToken($explodedToken) {
    return $explodedToken[1];
  }

  /**
   * Fixes JSON that has lost it's trailing brace.
   *
   * @param string $brokenJson A post exploded JSON that has lost it's trailing brace.
   *
   * @return string
   */
  private function fixJsonObject($brokenJson) {
    return $brokenJson . '}';
  }

  /**
   * Returns the 'aud' property of the supplied class.
   *
   * @param \stdClass $object An object that has an 'aud' property.
   *
   * @return string
   */
  private function getAudFromObject($object) {
    return $object->aud;
  }

  /**
   * Extracts the realm from the 'aud'.
   *
   * @param string $aud The 'aud' that holds the realm.
   *
   * @return string
   */
  private function extractRealmFromAud($aud) {
    return substr($aud, (strpos($aud, '@') + 1));
  }

  /**
   * Returns the a JSON encoded string that contains 'aud'.
   *
   * @return string
   */
  private function getJsonWithAud() {
    $jsonEncodedToken = $this->base64Decode($this->accessToken);
    $explodedToken = $this->explodeToken($jsonEncodedToken);
    $jsonWithAud = $this->findJsonWithAudInExplodedToken($explodedToken);
    $fixedSection = $this->fixJsonObject($jsonWithAud);

    return $fixedSection;
  }

  /**
   * Extracts the 'aud' from the access token.
   *
   * @return string
   */
  private function extractAud() {
    $fixedSection = $this->getJsonWithAud();
    $jsonDecoded = $this->jsonDecode($fixedSection);
    $aud = $this->getAudFromObject($jsonDecoded);

    return $aud;
  }

  /**
   * Extracts the realm from the access token.
   *
   * @return string
   */
  private function extractRealmFromToken() {
    $aud = $this->extractAud();
    $realm = $this->extractRealmFromAud($aud);

    return $realm;
  }

  /**
   * Converts the provided access token into a Token.
   *
   * @return TokenContract
   */
  public function convert() {
      $notBefore = "";
      $expiresOn = "";
      $realm = $this->extractRealmFromToken();

      $token = new Token($this->accessToken, $this->refreshToken, $notBefore, $expiresOn, $realm);

      return $token;
  }
}