<?php namespace Myriad\SharePoint;

use GuzzleHttp\Client;
use Illuminate\Contracts\Logging\Log;
use Myriad\SharePoint\Auth\SharePointOnline\Operations;
use Myriad\SharePoint\Auth\SharePointOnlineClaimAuthenticator;
use Myriad\SharePoint\Contracts;
use Myriad\SharePoint\Models\AuthenticatedUser;
use Myriad\SharePoint\Models\UnauthenticatedUser;
use Myriad\SharePoint\Models\Token;
use GuzzleHttp\Exception\BadResponseException;
use Myriad\SharePoint\Auth\AzureADAuthenticator;

/**
 * Class SharePointClient provides a gateway to SharePoint services.
 *
 * @author Chris Roemmich
 * @author David Kanenwisher
 * @package Myriad\SharePoint
 */
class SharePointClient implements Contracts\SharePointClient {

  /**
   * @var Client The guzzle client used for HTTP requests.
   */
  private $guzzle;

  /**
   * @var \Myriad\SharePoint\Contracts\Configuration The configuration for the SharePointClient.
   */
  private $config;

  /**
   * @var \Illuminate\Contracts\Logging\Log The object that handles logging messages.
   */
  private $log;

  /**
   * @var \Myriad\SharePoint\Auth\SharePointOnline\Operations The available authorization operations.
   */
  private $operations;

  /**
   * Creates a new SharePointClient.
   *
   * @param \Myriad\SharePoint\Contracts\Configuration $config The configuration object for the SharePointClient.
   * @param Client $guzzle The HTTP client used to make requests.
   * @param \Myriad\SharePoint\Auth\SharePointOnline\Operations $operations The available authorization operations.
   * @param \Illuminate\Contracts\Logging\Log $log The object that handles logging messages.
   */
  public function __construct(
    Contracts\Configuration $config,
    Client $guzzle,
    Operations $operations,
    Log $log
  ) {
    $this->config = $config;
    $this->guzzle = $guzzle;
    $this->log = $log;
    $this->operations = $operations;
  }

  /**
   * Converts a SharePointAuth response to a user.
   *
   * @param Contracts\Models\Token $sharepointAuthResponse The response from the authorization object.
   *
   * @return \Myriad\SharePoint\Contracts\Models\AuthenticatableUser
   */
  private function convertToUser($sharepointAuthResponse) {
    return new AuthenticatedUser (
      new SharePointOnlineGateway(new SharePointOnlineConnection(
        new Tenant($this->getTenant()),
        $sharepointAuthResponse,
        $this->getGuzzle(),
        $this->config,
        $this->getSiteHost() . $this->getSubSite()
      ))
    );
  }


  /**
   * Returns the tenant of the site.
   *
   * @return string
   */
  private function getTenant() {
    return $this->config->getTenant();
  }

  /**
   * Assembles the URL that SharePoint should be reached at.
   *
   * @param array ...$paths The paths that should be included in the request url.
   *
   * @return string
   */
  public function getEndpoint(...$paths) {
    $endpoint = 'https://' . $this->getSiteHost();
    foreach ($paths as $path) {
      $endpoint .= '/' . $path;
    }

    return $endpoint;
  }

  /**
   * Returns the SharePoint sub-site.
   *
   * @return string
   */
  public function getSubSite() {
    return $this->config->getSubSite();
  }

  /**
   * Returns the host of the site.
   *
   * @return string
   */
  public function getSiteHost() {
    return $this->getTenant() . '.sharepoint.com';
  }

  /**
   * Returns a User from SharePoint.
   *
   * @param string $username The username to login with.
   * @param string $password The password associted with the username.
   *
   * @return \Myriad\SharePoint\Contracts\Models\AuthenticatableUser
   */
  public function getUser($username, $password) {
    try {
      $authenticator = new AzureADAuthenticator($this->guzzle, $this->config, $username, $password);
      $token = $authenticator->getAccessToken();
      return $this->convertToUser($token);
    } catch (BadResponseException $e) {
      $this->log->error(__METHOD__ . " " . __LINE__ . " Error getting user from sharepoint:");
      $this->log->error(__METHOD__ . " " . __LINE__ . " Exception Message: " . $e->getMessage());
      $this->log->error(__METHOD__ . " " . __LINE__ . " Stack Trace: " . $e->getTraceAsString());
      $this->log->error(__METHOD__ . " " . __LINE__ . " Body: " .  $e->getResponse());
      $this->log->error($e->getMessage());
      $this->log->error($e->getTraceAsString());
      return new UnauthenticatedUser();
    } catch (\Exception $e) {
      $this->log->error($e->getMessage());
      $this->log->error($e->getTraceAsString());

      return new UnauthenticatedUser();
    }
  }

  /**
   * Get a list of users from sharepoint
   * @param  string $accessToken The access token to use when requesting the list
   * @return array              An array of emails and names of users in sharepoint
   */
  public function getUserList($accessToken) {
    // $blacklist = [
    //   'svc_wz_k2coreservice@id.mastec.com',
    //   'svc_wz_k2sharepoint@id.mastec.com',
    //   '_wz-accountspayable@id.mastec.com',
    // ];
    $response = $this->guzzle->get(
                  $this->getEndpoint() . '/sites/wz-portal/news/_api/web/siteusers',
                  ['headers' => ['Authorization' => ('bearer ' . $accessToken ),
                                   "Accept" => "application/json"
                                ]
                  ]
                );
    $result = $response->json();
    $userList = [];
    foreach ($result['value'] as $spUser) {
      if(!is_array($spUser['UserId'])) {
        continue;
      }
      $user = [];
      $tmp = explode('|', $spUser['LoginName']);
      $user['email'] = end($tmp);
      $user['name'] = $spUser['Title'];
      // filter out items in a blacklist
      // if( in_array($user['email'], $blacklist)) {
      //   continue;
      // }
      if(strpos($user['email'], '@') === false) {
        continue;
      }

      $userList[] = $user;
    }
    return $userList;
  }

  /**
   *
   *
   * @param string $accessToken The access token.
   * @param string $refreshToken The refresh token.
   *
   * @return \Myriad\SharePoint\Contracts\Models\AuthenticatableUser
   */
  public function getUserFromAccessTokenAndRefreshToken($accessToken, $refreshToken) {
    $converter = new Token\AccessTokenConverter($accessToken, $refreshToken);
    $token = $converter->convert();

    $user = $this->convertToUser($token);

    return $user;
  }

  /**
   * Return the guzzle instance used by the SharePointClient.
   *
   * @return Client
   */
  public function getGuzzle() {
    return $this->guzzle;
  }
}
