<?php namespace Myriad\SharePoint;

use Myriad\SharePoint\Contracts\Configuration as ConfigurationContract;
use Myriad\SharePoint\Contracts\Connection;
use Myriad\SharePoint\Contracts\Models\Token as TokenContract;
use Myriad\SharePoint\Contracts\Tenant;
use Myriad\SharePoint\Models\Token\Token;
use Illuminate\Support\Facades\Log;

/**
 * Class SharePointOnlineConnection is responsible for making requests to SharePointOnline.
 *
 * @author David Kanenwisher
 * @package Myriad\SharePoint
 */
class SharePointOnlineConnection implements Connection {

  /**
   * @var \Myriad\SharePoint\Contracts\Tenant The SharePointOnline Tenant.
   */
  private $tenant;

  /**
   * @var \Myriad\SharePoint\Contracts\Models\Token The SharePointOnline Token.
   */
  private $token;

  /**
   * @var \GuzzleHttp\Client The HTTP Client used to access SharePointOnline services.
   */
  private $guzzle;

  /**
   * @var ConfigurationContract The configuration for the SharePointOnlinec connection.
   */
  private $config;

  /**
   * @var string The URL of the SharePointOnline site.
   */
  private $siteUrl;

  /**
   * @var Illuminate\Contracts\Logging\Log  A logging utility
   */
  private $log;


  /**
   * Creates a new object.
   *
   * @param Tenant $tenant The name of the tenant.
   * @param TokenContract $token A SharePointOnline Token.
   * @param \GuzzleHttp\Client $guzzle The HTTP client used to talk to SharePointOnline.
   * @param ConfigurationContract $config The SharePoint configuration.
   * @param string $siteUrl The URL of the site.
   */
  public function __construct(Tenant $tenant, TokenContract $token, \GuzzleHttp\Client $guzzle, ConfigurationContract $config, $siteUrl) {
    $this->tenant = $tenant;
    $this->token = $token;
    $this->guzzle = $guzzle;
    $this->config = $config;
    $this->siteUrl = $siteUrl;
  }

  /**
   * Returns the SharePointOnlineURL.
   *
   * @return string
   */
  private function getSharePointOnlineUrl() {
    return "https://" . $this->siteUrl;
  }

  /**
   * Returns the access token.
   *
   * @return string
   */
  public function getAccessToken() {
    return $this->token->getAccessToken();
  }

  /**
   * Returns the refresh token.
   *
   * @return string
   */
  public function getRefreshToken() {
    return $this->token->getRefreshToken();
  }

  /**
   * Make a get request to SharePointOnline.
   *
   * @param string $resource The resource to request.
   * @param string $site The SharePoint Online site to make the request to.
   *
   * @return \GuzzleHttp\Message\Response
   */
  public function get($resource, $site = '') {
    $response = $this->guzzle->get($this->getSharePointOnlineUrl() . $site . "/_api/web/" . $resource,
      ['headers' => ['Authorization' => ('bearer ' . $this->getAccessToken() ),
        "Accept" => "application/json;odata=verbose"]]
    );

    return $response;
  }

  /**
   * Make a request to the SharePointOnline search API.
   *
   * @param string $url The url to append to the search request.
   *
   * @return \GuzzleHttp\Message\Response
   */
  public function search($url) {
    $response = $this->guzzle->get($this->getSharePointOnlineUrl() . "/_api/search/" . $url,
      ['headers' => ['Authorization' => ('bearer ' . $this->getAccessToken() ),
        "Accept" => "application/json;odata=verbose"]]
    );

    return $response;
  }

  /**
   * Checks to see if the tokens returned from SharePointOnline are invalid.
   *
   * @param string $tokens The tokens returned from SharePointOnline.
   *
   * @return bool
   */
  private function returnedTokensAreInvalid($tokens) {
    $is_invalid = false;

    if( is_null($tokens) ) {
      $is_invalid = true;
    }

    $required_keys = ['access_token', 'refresh_token', 'not_before', 'expires_on'];
    foreach ($required_keys as $key) {
      if( !array_key_exists($key, $tokens) ) {
        $is_invalid = true;
      }
    }

    return $is_invalid;
  }

  /**
   * Refreshes the connection's access token.
   *
   * @return bool
   */
  public function refreshAccessToken() {
    $tenant = $this->config->getTenant();
    $sharePointUrl = "https://{$tenant}.sharepoint.com";
    $tokenUrl = $this->config->getOAuth2TokenUrl();

    $body = [
      'resource' => $sharePointUrl,
      'client_id' => $this->config->getOAuth2ClientID(),
      'client_secret' => $this->config->getOAuth2ClientSecret(),
      'grant_type' => 'refresh_token',
      'refresh_token' => $this->getRefreshToken()
    ];
    $response = $this->guzzle->post($tokenUrl, ['body' => $body, 'allow_redirects' => false, 'exceptions' => false]);

    $tokens = $response->json();

    if ($this->returnedTokensAreInvalid($tokens)) {
      log::error(__METHOD__ . " " . __LINE__ . " Invalid token in response.  Status code: " . $response->getStatusCode() );
      log::error(__METHOD__ . " " . __LINE__ . " Body: " . $response->getBody() );
      return false;
    }

    $this->token = new Token(
      $tokens['access_token'],
      $tokens['refresh_token'],
      $tokens['not_before'],
      $tokens['expires_on'],
      ""
    );

    return true;
  }

  /**
   * Whether or not the token is authenticated.
   *
   * @return bool
   */
  public function isAuthenticated() {
    return $this->token->isValid();
  }

  /**
   * Returns the expires in time.
   *
   * @return string
   */
  public function getExpiresOn() {
    return $this->token->getExpiresOn();
  }
}
