<?php namespace Myriad\SharePoint;

use Myriad\SharePoint\Contracts\Connection;
use Myriad\SharePoint\Contracts\Gateway;
use Myriad\SharePoint\Models\CurrentUser;

/**
 * Class SharePointOnlineGateway is responsible for making requests to SharePointOnline.
 *
 * @author David Kanenwisher
 * @package Myriad\SharePoint
 */
class SharePointOnlineGateway implements Gateway {

  /**
   * @var \Myriad\SharePoint\Contracts\Connection The authenticated connection to SharePointOnline.
   */
  private $connection;

  /**
   * Create a new object.
   *
   * @param \Myriad\SharePoint\Contracts\Connection $connection An authenticated connection to SharePointOnline.
   */
  public function __construct(Connection $connection) {
    $this->connection = $connection;
  }

  /**
   * Retrieves information about the currently authenticated user from SharePointOnline.
   *
   * @return CurrentUser
   */
  public function getCurrentUser() {
    $response = $this->connection->get('CurrentUser');

    return new CurrentUser($response->json()['d']['Title'], $response->json()['d']['Email'], $response->json()['d']['LoginName']);
  }

  /**
   * Whether or not the token is authenticated.
   *
   * @return bool
   */
  public function isAuthenticated() {
    return $this->connection->isAuthenticated();
  }

  /**
   * Get the access token that this gateway is using to make requests.
   *
   * @return string
   */
  public function getAccessToken() {
    return $this->connection->getAccessToken();
  }

  /**
   * Returns the user's refresh token.
   *
   * @return string
   */
  public function getRefreshToken() {
    return $this->connection->getRefreshToken();
  }

  /**
   * Returns the user's expires on time.
   *
   * @return string
   */
  public function getExpiresOn() {
    return $this->connection->getExpiresOn();
  }

  /**
   * Refreshes the access token.
   *
   * @return bool
   */
  public function refreshAccessToken() {
    return $this->connection->refreshAccessToken();
  }

  /**
   * Make a get request to SharePointOnline.
   *
   * @param string $resource The resource to request.
   * @param string $site The SharePoint Online site to make the request to.
   *
   * @return \GuzzleHttp\Message\Response
   */
  public function get($resource, $site = '') {
    return $this->connection->get($resource, $site);
  }

  /**
   * Make a request to the SharePointOnline search API.
   *
   * @param string $url The url to append to the search request.
   *
   * @return \GuzzleHttp\Message\Response
   */
  public function search($url) {
    return $this->connection->search($url);
  }
}