<?php namespace Myriad\SharePoint;

use Illuminate\Contracts\Auth\Authenticatable;
use Illuminate\Contracts\Auth\UserProvider;

/**
 * Class SharePointUserProvider responsible for determining if a user has been authenticated to SharePoint.
 *
 * @author David Kanenwisher
 * @package Myriad\SharePoint
 */
class SharePointUserProvider implements UserProvider {

  /**
   * @var \Myriad\SharePoint\SharePointClient
   */
  private $sharePointClient;

  /**
   * @var Authenticatable The user that can be authenticated.
   */
  private $model;

  public function __construct(SharePointClient $sharePointClient, Authenticatable $model) {
    $this->sharePointClient = $sharePointClient;
    $this->model = $model;
  }

  /**
   * Retrieve a user by their unique identifier.
   *
   * @param  mixed $identifier
   *
   * @return \Illuminate\Contracts\Auth\Authenticatable|null
   */
  public function retrieveById($identifier) {
    return $this->model->newQuery()->find($identifier);
  }

  /**
   * Retrieve a user by their unique identifier and "remember me" token.
   *
   * @param  mixed $identifier
   * @param  string $token
   *
   * @return \Illuminate\Contracts\Auth\Authenticatable|null
   */
  public function retrieveByToken($identifier, $token) {
    $model = $this->model;

    return $model->newQuery()
                 ->where($model->getKeyName(), $identifier)
                 ->where($model->getRememberTokenName(), $token)
                 ->first();
  }

  /**
   * Update the "remember me" token for the given user in storage.
   *
   * @param  \Illuminate\Contracts\Auth\Authenticatable $user
   * @param  string $token
   *
   * @return void
   */
  public function updateRememberToken(Authenticatable $user, $token) {
    $user->setRememberToken($token);

    $user->save();
  }

  /**
   * Retrieve a user by the given credentials.
   *
   * @param  array $credentials
   *
   * @return \Illuminate\Contracts\Auth\Authenticatable|null
   */
  public function retrieveByCredentials(array $credentials) {
    $emailAddress = $credentials['email'];

    return $this->model->findOnlyOneUserByEmailAddressOrCreateANewUser($emailAddress);
  }

  /**
   * Validate a user against the given credentials.
   *
   * @param  \Illuminate\Contracts\Auth\Authenticatable $user
   * @param  array $credentials
   *
   * @return bool
   */
  public function validateCredentials(Authenticatable $user, array $credentials) {
    $sharePointUser = $this->sharePointClient->getUser($credentials['email'], $credentials['password']);

    if ($sharePointUser->isAuthenticated()) {

      $user->mergeSharePointUser($sharePointUser);

      return true;
    }

    return false;
  }
}